<h3>Useful Links</h3>
<ul>
  <li><a href="https://launchpad.net/murano">Murano @ Launchpad</a></li>
  <li><a href="https://wiki.openstack.org/wiki/Murano">Murano @ OpenStack Wiki</a></li>
</ul>

{% if READTHEDOCS %}
<script type='text/javascript'>
    $('div.body').css('margin', 0)
</script>
{% endif %}
