#    Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc


class MuranoPackageLoader(object, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def load_package(self, package_name, version_spec):
        pass

    @abc.abstractmethod
    def load_class_package(self, class_name, version_spec):
        pass

    @abc.abstractmethod
    def register_package(self, package):
        pass

    @abc.abstractmethod
    def import_fixation_table(self, fixations):
        pass

    @abc.abstractmethod
    def export_fixation_table(self):
        pass

    @abc.abstractmethod
    def compact_fixation_table(self):
        pass
