# coding: utf-8
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from murano.dsl import dsl_exception
from murano.tests.unit.dsl.foundation import object_model as om
from murano.tests.unit.dsl.foundation import test_case


class TestUnicode(test_case.DslTestCase):
    def setUp(self):
        super(TestUnicode, self).setUp()
        self._runner = self.new_runner(om.Object('TestUnicode'))

    def test_literal(self):
        self.assertEqual(
            u"солнце ♥ φεγγάρι",
            self._runner.testLiteral())

    def test_expression(self):
        self.assertEqual(
            u"СОЛНЦЕ ♥ ΦΕΓΓΆΡΙ",
            self._runner.testExpression())

    def test_parameter(self):
        self.assertEqual(
            u"СОЛНЦЕ ♥ ΦΕΓΓΆΡΙ",
            self._runner.testParameter())

    def test_exception(self):
        x = self.assertRaises(
            dsl_exception.MuranoPlException,
            self._runner.testException)
        self.assertEqual(u"солнце ♥ φεγγάρι", x.message)
